# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: RobotPose.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import CommonTypes_pb2 as CommonTypes__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='RobotPose.proto',
  package='naothmessages',
  syntax='proto2',
  serialized_pb=_b('\n\x0fRobotPose.proto\x12\rnaothmessages\x1a\x11\x43ommonTypes.proto\"\xb5\x01\n\tRobotPose\x12#\n\x04pose\x18\x01 \x02(\x0b\x32\x15.naothmessages.Pose2D\x12\x0f\n\x07isValid\x18\x02 \x01(\x08\x12\x38\n\x12principleAxisMajor\x18\x03 \x01(\x0b\x32\x1c.naothmessages.DoubleVector2\x12\x38\n\x12principleAxisMinor\x18\x04 \x01(\x0b\x32\x1c.naothmessages.DoubleVector2B\x1b\n\x19\x64\x65.naoth.rc.core.messages')
  ,
  dependencies=[CommonTypes__pb2.DESCRIPTOR,])




_ROBOTPOSE = _descriptor.Descriptor(
  name='RobotPose',
  full_name='naothmessages.RobotPose',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='pose', full_name='naothmessages.RobotPose.pose', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='isValid', full_name='naothmessages.RobotPose.isValid', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='principleAxisMajor', full_name='naothmessages.RobotPose.principleAxisMajor', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='principleAxisMinor', full_name='naothmessages.RobotPose.principleAxisMinor', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=54,
  serialized_end=235,
)

_ROBOTPOSE.fields_by_name['pose'].message_type = CommonTypes__pb2._POSE2D
_ROBOTPOSE.fields_by_name['principleAxisMajor'].message_type = CommonTypes__pb2._DOUBLEVECTOR2
_ROBOTPOSE.fields_by_name['principleAxisMinor'].message_type = CommonTypes__pb2._DOUBLEVECTOR2
DESCRIPTOR.message_types_by_name['RobotPose'] = _ROBOTPOSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

RobotPose = _reflection.GeneratedProtocolMessageType('RobotPose', (_message.Message,), dict(
  DESCRIPTOR = _ROBOTPOSE,
  __module__ = 'RobotPose_pb2'
  # @@protoc_insertion_point(class_scope:naothmessages.RobotPose)
  ))
_sym_db.RegisterMessage(RobotPose)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\n\031de.naoth.rc.core.messages'))
# @@protoc_insertion_point(module_scope)




def load_data(type:str=None):
    from . import __get_file
    base = 'https://www2.informatik.hu-berlin.de/~naoth/ressources/log/demo_image/'
    # taken from /vol/repl261-vol4/naoth/logs/2019-11-21_Koeln/
    # 2019-11-21_16-20-00_Berlin United_vs_Nao_Devils_Dortmund_half1/game_logs/1_96_Nao0377_191122-0148
    files = {
        'image': ('test.png', '97a053f4859e2850ca768f8e6f82d35a'),
        'image_log': ('images.log', '9550c35fc831d99d031455fd305d7494'),
        'game_log': ('game.log', '2e088b23039c5d83b70dabea8b658390'),
        'ball': ('rc17_ball_far.log', 'e85f5ed1ea5777aaf01049895481be70'),
    }

    if type:
        return __get_file(files[type][0], file_hash=files[type][1], origin=base, cache_subdir='demo_image')

    return {k: __get_file(v[0], file_hash=v[1], origin=base, cache_subdir='demo_image') for k, v in files.items()}



"""Representation of the SPL standard message."""
from struct import Struct as _Struct

from .pb import TeamMessage_pb2 as _TM
from .math import Vector2 as _v2, Pose2D as _p2


class MixedTeamMessage(_Struct):

    def __init__(self, data=None):
        """Constructor."""
        super(MixedTeamMessage, self).__init__('Q4B')
        # based on the given data the message is initialized with default values or parsed from the data string
        if data is not None:
            self.unpack(data)
        else:
            self.setDefaults()

    def setDefaults(self):
        self.timestamp = 0
        self.teamID = 4
        self.isPenalized = 0
        self.whistleDetected = 0
        self.dummy = 42

    def unpack(self, data):
        # check 'data' length
        if len(data) < self.size:
            raise Exception("Invalid data size for MixedTeamMessage")

        msg = _Struct.unpack(self, data[:self.size])
        it = iter(msg)
        self.timestamp = next(it)
        self.teamID = next(it)
        self.isPenalized = next(it)
        self.whistleDetected = next(it)
        self.dummy = next(it)

    def pack(self):
        return _Struct.pack(self,
                            self.timestamp,
                            self.teamID,
                            self.isPenalized,
                            self.whistleDetected,
                            self.dummy
                            )

    def __str__(self):
        return str(self.__dict__)


class SPLMessage(_Struct):
    """Representation of the standard SPLMessage."""

    SPL_STANDARD_MESSAGE_STRUCT_HEADER = b'SPL '
    SPL_STANDARD_MESSAGE_STRUCT_VERSION = 7
    SPL_STANDARD_MESSAGE_DATA_SIZE = 474
    SPL_STANDARD_MESSAGE_MAX_NUM_OF_PLAYERS = 6

    def __init__(self, teamnumber: int = 0, playernumber: int = 0, data=None):
        """Constructor."""
        # 4s    header +
        # 3b    version + playernumber + teamnumber
        # ?     fallen
        # 6f    pose_x, pose_y, pose_r, ballage, ball_x, ball_y
        # h     size
        super(SPLMessage, self).__init__('4s3b?6fh')
        # based on the given data the message is initialized with default values or parsed from the data string
        if data is not None:
            self.unpack(data)
        else:
            self.setDefaults(teamnumber, playernumber)

    def setDefaults(self, teamnumber=0, playernumber=0):

        self.playerNumber = playernumber
        self.teamNumber = teamnumber
        self.fallen = False
        self.pose = _p2(_v2(0.0, 0.0), 0.0)  # x, y, r | +/-4500, +/-3000
        # self.walkingTo = _v2(0.0, 0.0)
        # self.shootingTo = _v2(0.0, 0.0)
        self.ballAge = -1
        self.ballPosition = _v2(0.0, 0.0)
        # self.ballVelocity = _v2(0.0, 0.0)
        # self.suggestion = [0 for x in range(self.SPL_STANDARD_MESSAGE_MAX_NUM_OF_PLAYERS)]
        # self.intention = 0
        # self.averageWalkSpeed = 200  # see TeamCommSender
        # self.maxKickDistance = 3000  # see TeamCommSender
        # self.currentPositionConfidence = 100
        # self.currentSideConfidence = 100

        self._mixed = MixedTeamMessage()

        self.data = _TM.BUUserTeamMessage()

        # set known default values of custom message part
        for field in self.data.DESCRIPTOR.fields:
            if field.has_default_value:
                setattr(self.data, field.name, field.default_value)

        self.numOfDataBytes = self.data.ByteSize() + self._mixed.size

    def pack(self):

        return _Struct.pack(self,
                            self.SPL_STANDARD_MESSAGE_STRUCT_HEADER,
                            self.SPL_STANDARD_MESSAGE_STRUCT_VERSION,
                            self.playerNumber,
                            self.teamNumber,
                            self.fallen,
                            self.pose.translation.x, self.pose.translation.y, self.pose.rotation,
                            # *self.walkingTo.__dict__.values(),
                            # *self.shootingTo.__dict__.values(),
                            self.ballAge,
                            # *self.ballPosition.__dict__.values(),
                            self.ballPosition.x,
                            self.ballPosition.y,
                            # *self.ballVelocity.__dict__.values(),
                            # *self.suggestion,
                            # self.intention,
                            # self.averageWalkSpeed,
                            # self.maxKickDistance,
                            # self.currentPositionConfidence,
                            # self.currentSideConfidence,
                            (self.data.ByteSize() + self._mixed.size)
                            ) + self._mixed.pack() \
               + self.data.SerializeToString()

    def unpack(self, data):
        # check 'data' length
        if len(data) < self.size:
            raise ErrorNotAnSplMessage("Not an SPL Message.")

        msg = _Struct.unpack(self, data[:self.size])

        # check header
        if msg[0] != self.SPL_STANDARD_MESSAGE_STRUCT_HEADER:
            raise ErrorInvalidSplHeader("Invalid header! (" + msg[0].decode("utf-8") + ")")

        # check spl message version
        if msg[1] != self.SPL_STANDARD_MESSAGE_STRUCT_VERSION:
            raise ErrorWrongSplMessageVersion("Wrong version: received " + str(msg[1]) + ", but expected " + str(
                self.SPL_STANDARD_MESSAGE_STRUCT_VERSION))

        # assign data
        it = iter(msg[2:])
        self.playerNumber = next(it)
        self.teamNumber = next(it)
        self.fallen = next(it)
        self.pose = _p2(_v2(next(it), next(it)), next(it))
        # self.walkingTo = _v2( next(it), next(it) )
        # self.shootingTo = _v2( next(it), next(it) )
        self.ballAge = next(it)
        self.ballPosition = _v2(next(it), next(it))
        # self.ballVelocity = _v2( next(it), next(it) )
        # self.suggestion = [ next(it) for i in range(self.SPL_STANDARD_MESSAGE_MAX_NUM_OF_PLAYERS) ]
        # self.intention = next(it)
        # self.averageWalkSpeed = next(it)
        # self.maxKickDistance = next(it)
        # self.currentPositionConfidence = next(it)
        # self.currentSideConfidence = next(it)
        self.numOfDataBytes = next(it)
        self.data = data[self.size:self.size + self.numOfDataBytes]

        # unpack mixed team part
        self._mixed = MixedTeamMessage()
        self._mixed.unpack(self.data[:self._mixed.size])

        # check if we got a 'valid' mixed team message and adjust data offset
        offset = self._mixed.size if self._mixed.teamID == 4 else 0

        custom = None
        try:
            custom = _TM.BUUserTeamMessage()
            custom.ParseFromString(data[self.size + offset:self.size + offset + self.numOfDataBytes])
        except:
            # if we can't parse custom data - it is not our message
            pass
        finally:
            # if its our custom data ...
            if custom is not None and custom.ByteSize() > 0 and (custom.HasField('key') and custom.key == "naoth"):
                self.data = custom

    def parseFromLogJson(self, data):
        self.__map_json_data('playerNum', 'playerNumber', data)
        self.__map_json_data('teamNum', 'teamNumber', data)
        self.__map_json_data('fallen', 'fallen', data)
        self.__map_json_data((self.pose, 'x'), 'pose_x', data)
        self.__map_json_data((self.pose, 'y'), 'pose_y', data)
        self.__map_json_data((self.pose, 'r'), 'pose_a', data)
        self.__map_json_data((self.walkingTo, 'x'), 'walkingTo_x', data)
        self.__map_json_data((self.walkingTo, 'y'), 'walkingTo_y', data)
        self.__map_json_data((self.shootingTo, 'x'), 'shootingTo_x', data)
        self.__map_json_data((self.shootingTo, 'y'), 'shootingTo_y', data)
        self.__map_json_data('ballAge', 'ballAge', data)
        self.__map_json_data((self.ball, 'x'), 'ball_x', data)
        self.__map_json_data((self.ball, 'y'), 'ball_y', data)
        self.__map_json_data((self.ballVel, 'x'), 'ballVel_x', data)
        self.__map_json_data((self.ballVel, 'y'), 'ballVel_y', data)
        self.__map_json_data('suggestion', 'suggestion', data)
        self.__map_json_data('intention', 'intention', data)
        self.__map_json_data('averageWalkSpeed', 'averageWalkSpeed', data)
        self.__map_json_data('maxKickDistance', 'maxKickDistance', data)
        self.__map_json_data('currentPositionConfidence', 'currentPositionConfidence', data)
        self.__map_json_data('currentSideConfidence', 'currentSideConfidence', data)
        self.__map_json_data('numOfDataBytes', 'numOfDataBytes', data)
        self.__map_json_data('data', 'data', data,
                             lambda x: bytes(list(map(lambda i: -128 * (i // 128) + (i % 128), x))))

    def __map_json_data(self, var, key, data, func=lambda x: x):
        if key in data:
            if type(var) == tuple:
                var[0].__dict__[var[1]] = func(data[key])
            else:
                self.__dict__[var] = func(data[key])

    def __repr__(self):
        """Returns all 'active' message fields as string."""
        result = ""
        for attr in self.__dict__:
            # print(attr, isinstance(self.__dict__[attr], bytes), self.__dict__[attr])
            if attr == "data" and not isinstance(self.__dict__[attr], bytes):
                fields = self.__dict__[attr].DESCRIPTOR.fields_by_name
                for custom_attr in fields:
                    # do not show deprecated custom fields!
                    if fields[custom_attr].GetOptions().deprecated:
                        continue
                    result += "\t" + custom_attr + " = " + str(getattr(self.__dict__[attr], custom_attr)) + "\n"
            elif attr.startswith("_"):
                # skip attributes starting with '_' (private)
                continue
            else:
                result += "\t" + attr + " = " + str(self.__dict__[attr]) + "\n"
        return result


class ErrorNotAnSplMessage(Exception):
    """Exception, if given data is not an SPL message."""
    pass


class ErrorWrongSplMessageVersion(Exception):
    """Wrong spl message version exception."""
    pass


class ErrorInvalidSplHeader(Exception):
    """Wrong spl message header exception."""
    pass



import math as _math


class Vector3:
    def __init__(self, x=0, y=0, z=0):
        self.x = x
        self.y = y
        self.z = z

    def __add__(self, other):
        return Vector3(self.x + other.x, self.y + other.y, self.z + other.z)

    def __sub__(self, other):
        return Vector3(self.x - other.x, self.y - other.y, self.z - other.z)

    def __neg__(self):
        return Vector3(-self.x, -self.y, -self.z)

    def abs(self):
        return _math.sqrt(self * self)

    def __mul__(self, other):
        if isinstance(other, Vector3):
            return self.x * other.x + self.y * other.y + self.z * other.z
        elif isinstance(other, (int, float)):
            return Vector3(self.x * other, self.y * other, self.z * other)
        else:
            return NotImplemented

    def __rmul__(self, other):
        if isinstance(other, Vector3):
            return self.x * other.x + self.y * other.y + self.z * other.z
        elif isinstance(other, (int, float)):
            return Vector3(self.x * other, self.y * other, self.z * other)
        else:
            return NotImplemented

    def __str__(self):
        return "({0},{1},{2})".format(self.x, self.y, self.z)

    def __repr__(self):
        return str(self)


class Matrix3x3:
    def __init__(self, c1=Vector3(), c2=Vector3(), c3=Vector3()):
        self.c1 = c1
        self.c2 = c2
        self.c3 = c3

    def __mul__(self, other):
        if isinstance(other, Matrix3x3):
            return Matrix3x3(self * other.c1, self * other.c2, self * other.c3)
        elif isinstance(other, Vector3):
            return self.c1 * other.x + self.c2 * other.y + self.c3 * other.z
        else:
            return NotImplemented

    def transpose(self):
        return Matrix3x3(
          Vector3(self.c1.x, self.c2.x, self.c3.x),
          Vector3(self.c1.y, self.c2.y, self.c3.y),
          Vector3(self.c1.z, self.c2.z, self.c3.z))

    @classmethod
    def eye(Matrix3x3):
        # the vectors are columnwise
        return Matrix3x3(
          Vector3(1, 0, 0),
          Vector3(0, 1, 0),
          Vector3(0, 0, 1)
        )

    def __str__(self):
        return "{0} {1} {2}\n{3} {4} {5}\n{6} {7} {8}".format(
          self.c1.x, self.c2.x, self.c3.x,
          self.c1.y, self.c2.y, self.c3.y,
          self.c1.z, self.c2.z, self.c3.z)

    def __repr__(self):
        return str(self)


class Pose3D:
    def __init__(self):
        self.translation = Vector3()
        self.rotation = Matrix3x3.eye()

    def __mul__(self, other):
        if isinstance(other, Vector3):
            return self.rotation * other + self.translation
        elif isinstance(other, Pose3D):
            p = Pose3D()
            p.translation = self * other.translation
            p.rotation = self.rotation * other.rotation
            return p
        else:
            return NotImplemented

    def __invert__(self):
        p = Pose3D()
        p.rotation = self.rotation.transpose()
        p.translation = p.rotation * (Vector3() - self.translation)
        return p

    def __str__(self):
        return "(translation = {0}, rotation = {1})".format(self.translation, self.rotation)

    def __repr__(self):
        return str(self)



import inspect as _inspect
import sys
from .. import pb as _pb
from ._xabsl import XABSLSymbols

# get all protobuf classes
_proto = {}
for _m in _inspect.getmembers(_pb, _inspect.ismodule):
    for _c in _inspect.getmembers(_m[1], _inspect.isclass):
        if not _c[0].startswith('_'):
            _proto[_c[0]] = _c[1]


class Parser:
    def __init__(self):
        self.nameToType = {}

    def register(self, name, type):
        self.nameToType[name] = type

    def parse(self, name, data):
        message = None

        if name in self.nameToType:
            name = self.nameToType[name]

        if name in _proto:
            message = _proto[name]()

        if message is not None:
            message.ParseFromString(data)

        return message


class BehaviorParser:
    def __init__(self, data=None):

        self.parser = Parser()
        self.symbols = XABSLSymbols()
        self.__options = None
        self.__current_options = None

        self.init(data)

    def init(self, data):
        if data is not None:
            # check given data
            if data == b'unknown representation\n':
                raise Exception('Unknown representation')

            message = self.parser.parse('BehaviorStateComplete', data)

            # initialize options
            self.__options = message.options

            # initialize symbols
            for s in message.inputSymbolList.decimal:
                self.symbols.values[s.name] = s.value
                self.symbols.decimalIdToName[s.id] = s.name

            for s in message.inputSymbolList.boolean:
                self.symbols.values[s.name] = s.value
                self.symbols.booleanIdToName[s.id] = s.name

            for s in message.inputSymbolList.enumerated:
                self.symbols.values[s.name] = s.value
                self.symbols.enumIdToName[s.id] = s.name

            return message

    def __check_initialized(self):
        if self.__options is None:
            # print(" BehaviorParser must be initialized with 'BehaviorStateComplete' data! -> Running Init")
            return False

        return True

    def parse(self, name, data):
        if not self.__check_initialized() and name == "BehaviorStateComplete":
            message = self.init(data)

            return (message, self.__current_options, self.symbols)
        elif name == "BehaviorStateSparse":
            self.__current_options = {}

            message = self.parser.parse("BehaviorStateSparse", data)

            # process active options
            for o in message.activeRootActions:
                self.__parseOption(o)

            # process symbols
            for s in message.inputSymbolList.decimal:
                self.symbols.decimal(s.id, s.value)

            for s in message.inputSymbolList.boolean:
                self.symbols.boolean(s.id, s.value)

            for s in message.inputSymbolList.enumerated:
                self.symbols.enum(s.id, s.value)

            return (message, self.__current_options, self.symbols)
        else:
            return(self.parser.parse(name, data))


    def __parseOption(self, o):
        if o.type == 0:  # Option
            optionComplete = self.__options[o.option.id]
            self.__current_options[optionComplete.name] = {
                'time': o.option.timeOfExecution,
                'state': optionComplete.states[o.option.activeState],
                'stateTime': o.option.stateTime
            }

            for so in o.option.activeSubActions:
                self.__parseOption(so)

    def getActiveOptions(self):
        return list(self.__current_options.keys())

    def isActiveOption(self, option):
        return option in self.__current_options

    def getActiveStates(self):
        return [s['state'].name for s in self.__current_options.values()]

    def getActiveOptionState(self, option):
        if option in self.__current_options:
            return self.__current_options[option]['state'].name

    def __str__(self):
        # TODO: how to represent the options?!
        result = str(self.symbols) + ',\n[\n'
        if self.__current_options:
            for o in self.__current_options:
                result += '\t{} [{} ms] - {} [{} ms]\n'.format(o, self.__current_options[o]['time'],
                                                               self.__current_options[o]['state'].name,
                                                               self.__current_options[o]['stateTime'])
        result += ']'

        return result




def __validate_file(file: str, file_hash: str):
    from hashlib import md5
    fhash = md5()
    with open(file, 'rb') as f:
        fb = f.read(65536)
        while len(fb) > 0:
            fhash.update(fb)
            fb = f.read(65536)
    return file_hash == fhash.hexdigest()


def __get_file(name: str,
               file_hash: str = None,
               cache_subdir: str = None,
               origin: str = 'https://www2.informatik.hu-berlin.de/~naoth/ressources/log/'):
    import os
    from urllib.request import urlretrieve
    from urllib.parse import urljoin, quote
    from urllib.error import HTTPError, URLError

    # prepare cache dir
    cache_dir = os.path.join(os.path.expanduser('~'), '.naoth', 'datasets')

    # use subdir?
    if cache_subdir:
        cache_dir = os.path.join(cache_dir, cache_subdir)

    if not os.path.exists(cache_dir):
        os.makedirs(cache_dir)

    file = os.path.join(cache_dir, name)

    # check, if we need to (re-)download the file
    download = False
    if os.path.isfile(file):
        if file_hash is not None and not __validate_file(file, file_hash):
            print('A local file was found, but it seems to be incomplete or outdated, so we will re-download the data.')
            download = True
    else:
        download = True

    # download the actual file
    if download:
        origin_file = urljoin(origin, quote(name))
        print('Downloading data from', origin_file)

        def dl_progress(count, block_size, total_size):
            print('\r', 'Progress: {0:.2%}'.format(min((count * block_size) / total_size, 1.0)), sep='', end='', flush=True)

        error_msg = 'URL fetch failure on {} : {} -- {}'
        try:
            try:
                urlretrieve(origin_file, file, dl_progress)
                print('\nFinished')
            except HTTPError as e:
                raise Exception(error_msg.format(origin_file, e.code, e.reason))
            except URLError as e:
                raise Exception(error_msg.format(origin_file, e.errno, e.reason))
        except (Exception, KeyboardInterrupt):
            if os.path.exists(file):
                os.remove(file)
            raise

    return file


from . import motion
from . import fsr
from . import edgels
from . import joints
from . import images



import enum as _enum


class XABSLState:
    def __init__(self, state):
        self.name = state.name
        self.target = state.target

    def __str__(self):
        return self.name


class XABSLOption:
    def __init__(self, time, state, state_time):
        self.time = time
        self.state = XABSLState(state)
        self.state_time = state_time

        self.symbols = {}
        self.parameters = {}

        self.sub_options = {}

    def __getitem__(self, option_name):
        return self.sub_options[option_name]

    def __str__(self):
        return f'[{self.time} ms] {self.state} [{self.state_time} ms]'

    @staticmethod
    def option_to_string(name, option, level=0):
        """
        Creates a BehaviorState string similar to the output of RobotControl
        :param name: root option name
        :param option: root option
        :return:
        """
        padding = level * '\t'

        string = f'{padding}{name} {option}'

        if option.parameters:
            string += f'\n{padding}\t' + f'\n{padding}\t'.join(f'@{name}={value}' for name, value in option.parameters.items())

        if option.symbols:
            string += f'\n{padding}\t' + f'\n{padding}\t'.join(f'{name}={value}' for name, value in option.symbols.items())

        if option.sub_options:
            string += f'\n' + '\n'.join(XABSLOption.option_to_string(sub_option_name, sub_option, level + 1)
                                        for sub_option_name, sub_option in option.sub_options.items())
        return string


class XABSLAction(_enum.Enum):
    XABSLSymbol = _enum.auto()
    XABSLOption = _enum.auto()



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: Framework-Representations.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import CommonTypes_pb2 as CommonTypes__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='Framework-Representations.proto',
  package='naothmessages',
  syntax='proto2',
  serialized_pb=_b('\n\x1f\x46ramework-Representations.proto\x12\rnaothmessages\x1a\x11\x43ommonTypes.proto\"\xdb\x01\n\x05Image\x12\x0c\n\x04\x64\x61ta\x18\x01 \x02(\x0c\x12\x12\n\x05width\x18\x02 \x01(\r:\x03\x36\x34\x30\x12\x13\n\x06height\x18\x03 \x01(\r:\x03\x34\x38\x30\x12-\n\ncameraInfo\x18\x04 \x01(\x0b\x32\x19.naothmessages.CameraInfo\x12\x30\n\x06\x66ormat\x18\x05 \x01(\x0e\x32\x1b.naothmessages.Image.Format:\x03YUV\x12\x11\n\ttimestamp\x18\x06 \x01(\r\"\'\n\x06\x46ormat\x12\x07\n\x03YUV\x10\x00\x12\n\n\x06YUV422\x10\x01\x12\x08\n\x04JPEG\x10\x02\"\xd2\x04\n\nCameraInfo\x12\x17\n\x0fresolutionWidth\x18\x01 \x02(\r\x12\x18\n\x10resolutionHeight\x18\x02 \x02(\r\x12\x31\n\x08\x63\x61meraID\x18\x04 \x01(\x0e\x32\x17.naothmessages.CameraID:\x06\x62ottom\x12\x17\n\x0b\x66ocalLength\x18\x05 \x01(\x01\x42\x02\x18\x01\x12\x1d\n\x11openingAngleWidth\x18\x06 \x01(\x01\x42\x02\x18\x01\x12\x1e\n\x12openingAngleHeight\x18\x07 \x01(\x01\x42\x02\x18\x01\x12\x1a\n\x0eopticalCenterX\x18\x08 \x01(\x01\x42\x02\x18\x01\x12\x1a\n\x0eopticalCenterY\x18\t \x01(\x01\x42\x02\x18\x01\x12\x10\n\x04size\x18\n \x01(\x03\x42\x02\x18\x01\x12\x16\n\nmemorysize\x18\x0b \x01(\x03\x42\x02\x18\x01\x12\x1c\n\x10\x63\x61meraRollOffset\x18\x0c \x01(\x01\x42\x02\x18\x01\x12\x1c\n\x10\x63\x61meraTiltOffset\x18\r \x01(\x01\x42\x02\x18\x01\x12:\n\x10\x63orrectionOffset\x18\x12 \x03(\x0b\x32\x1c.naothmessages.DoubleVector2B\x02\x18\x01\x12\x39\n\x0fheadJointOffset\x18\x13 \x03(\x0b\x32\x1c.naothmessages.DoubleVector2B\x02\x18\x01\x12\x31\n\x0etransformation\x18\x0e \x03(\x0b\x32\x15.naothmessages.Pose3DB\x02\x18\x01\x12\x1c\n\x14openingAngleDiagonal\x18\x0f \x01(\x01\x12\x11\n\tpixelSize\x18\x10 \x01(\x01\x12\r\n\x05\x66ocus\x18\x11 \x01(\x01\"I\n\tJointData\x12\x10\n\x08position\x18\x01 \x03(\x01\x12\x11\n\tstiffness\x18\x02 \x03(\x01\x12\n\n\x02\x64p\x18\x03 \x03(\x01\x12\x0b\n\x03\x64\x64p\x18\x04 \x03(\x01\"l\n\x0fSensorJointData\x12+\n\tjointData\x18\x01 \x02(\x0b\x32\x18.naothmessages.JointData\x12\x13\n\x0btemperature\x18\x02 \x03(\x01\x12\x17\n\x0f\x65lectricCurrent\x18\x03 \x03(\x01\"i\n\x15UltraSoundReceiveData\x12\x1a\n\x12ultraSoundTimeStep\x18\x01 \x01(\r\x12\x0f\n\x07rawdata\x18\x02 \x02(\x01\x12\x10\n\x08\x64\x61taLeft\x18\x03 \x03(\x01\x12\x11\n\tdataRight\x18\x04 \x03(\x01\".\n\tFrameInfo\x12\x13\n\x0b\x66rameNumber\x18\x01 \x02(\r\x12\x0c\n\x04time\x18\x02 \x02(\r\"p\n\tRobotInfo\x12\x10\n\x08platform\x18\x01 \x01(\t\x12\x14\n\x0c\x62odyNickName\x18\x02 \x01(\t\x12\x14\n\x0cheadNickName\x18\x03 \x01(\t\x12\x0e\n\x06\x62odyID\x18\x04 \x01(\t\x12\x15\n\rbasicTimeStep\x18\x05 \x01(\r\"=\n\x07\x46SRData\x12\x11\n\x05\x66orce\x18\x01 \x03(\x01\x42\x02\x18\x01\x12\x0c\n\x04\x64\x61ta\x18\x02 \x03(\x01\x12\x11\n\x05valid\x18\x03 \x03(\x08\x42\x02\x18\x01\"\x8c\x01\n\x11\x41\x63\x63\x65lerometerData\x12\x1c\n\x10legacyPackedData\x18\x01 \x03(\x01\x42\x02\x18\x01\x12*\n\x04\x64\x61ta\x18\x02 \x01(\x0b\x32\x1c.naothmessages.DoubleVector3\x12-\n\x07rawData\x18\x03 \x01(\x0b\x32\x1c.naothmessages.DoubleVector3\"\x95\x01\n\rGyrometerData\x12\x1c\n\x10legacyPackedData\x18\x01 \x03(\x01\x42\x02\x18\x01\x12*\n\x04\x64\x61ta\x18\x02 \x01(\x0b\x32\x1c.naothmessages.DoubleVector3\x12-\n\x07rawData\x18\x03 \x01(\x0b\x32\x1c.naothmessages.DoubleVector3\x12\x0b\n\x03ref\x18\x04 \x01(\x01\"^\n\x12InertialSensorData\x12\x1c\n\x10legacyPackedData\x18\x01 \x03(\x01\x42\x02\x18\x01\x12*\n\x04\x64\x61ta\x18\x02 \x01(\x0b\x32\x1c.naothmessages.DoubleVector2\"\xd3\x02\n\nButtonData\x12\x11\n\tisPressed\x18\x01 \x03(\x08\x12\x1a\n\x12numOfFramesPressed\x18\x02 \x03(\r\x12\x14\n\x0c\x65ventCounter\x18\x03 \x03(\r\"\xff\x01\n\x08\x42uttonID\x12\t\n\x05\x43hest\x10\x00\x12\x10\n\x0cLeftFootLeft\x10\x01\x12\x11\n\rLeftFootRight\x10\x02\x12\x11\n\rRightFootLeft\x10\x03\x12\x12\n\x0eRightFootRight\x10\x04\x12\r\n\tHeadFront\x10\x05\x12\x0e\n\nHeadMiddle\x10\x06\x12\x0c\n\x08HeadRear\x10\x07\x12\x10\n\x0cLeftHandBack\x10\x08\x12\x10\n\x0cLeftHandLeft\x10\t\x12\x11\n\rLeftHandRight\x10\n\x12\x11\n\rRightHandBack\x10\x0b\x12\x11\n\rRightHandLeft\x10\x0c\x12\x12\n\x0eRightHandRight\x10\rB\x1b\n\x19\x64\x65.naoth.rc.core.messages')
  ,
  dependencies=[CommonTypes__pb2.DESCRIPTOR,])



_IMAGE_FORMAT = _descriptor.EnumDescriptor(
  name='Format',
  full_name='naothmessages.Image.Format',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='YUV', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='YUV422', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='JPEG', index=2, number=2,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=250,
  serialized_end=289,
)
_sym_db.RegisterEnumDescriptor(_IMAGE_FORMAT)

_BUTTONDATA_BUTTONID = _descriptor.EnumDescriptor(
  name='ButtonID',
  full_name='naothmessages.ButtonData.ButtonID',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='Chest', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LeftFootLeft', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LeftFootRight', index=2, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='RightFootLeft', index=3, number=3,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='RightFootRight', index=4, number=4,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='HeadFront', index=5, number=5,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='HeadMiddle', index=6, number=6,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='HeadRear', index=7, number=7,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LeftHandBack', index=8, number=8,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LeftHandLeft', index=9, number=9,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LeftHandRight', index=10, number=10,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='RightHandBack', index=11, number=11,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='RightHandLeft', index=12, number=12,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='RightHandRight', index=13, number=13,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=1881,
  serialized_end=2136,
)
_sym_db.RegisterEnumDescriptor(_BUTTONDATA_BUTTONID)


_IMAGE = _descriptor.Descriptor(
  name='Image',
  full_name='naothmessages.Image',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='data', full_name='naothmessages.Image.data', index=0,
      number=1, type=12, cpp_type=9, label=2,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='width', full_name='naothmessages.Image.width', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=True, default_value=640,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='height', full_name='naothmessages.Image.height', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=True, default_value=480,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='cameraInfo', full_name='naothmessages.Image.cameraInfo', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='format', full_name='naothmessages.Image.format', index=4,
      number=5, type=14, cpp_type=8, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='naothmessages.Image.timestamp', index=5,
      number=6, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _IMAGE_FORMAT,
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=70,
  serialized_end=289,
)


_CAMERAINFO = _descriptor.Descriptor(
  name='CameraInfo',
  full_name='naothmessages.CameraInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='resolutionWidth', full_name='naothmessages.CameraInfo.resolutionWidth', index=0,
      number=1, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='resolutionHeight', full_name='naothmessages.CameraInfo.resolutionHeight', index=1,
      number=2, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='cameraID', full_name='naothmessages.CameraInfo.cameraID', index=2,
      number=4, type=14, cpp_type=8, label=1,
      has_default_value=True, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='focalLength', full_name='naothmessages.CameraInfo.focalLength', index=3,
      number=5, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\030\001'))),
    _descriptor.FieldDescriptor(
      name='openingAngleWidth', full_name='naothmessages.CameraInfo.openingAngleWidth', index=4,
      number=6, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\030\001'))),
    _descriptor.FieldDescriptor(
      name='openingAngleHeight', full_name='naothmessages.CameraInfo.openingAngleHeight', index=5,
      number=7, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\030\001'))),
    _descriptor.FieldDescriptor(
      name='opticalCenterX', full_name='naothmessages.CameraInfo.opticalCenterX', index=6,
      number=8, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\030\001'))),
    _descriptor.FieldDescriptor(
      name='opticalCenterY', full_name='naothmessages.CameraInfo.opticalCenterY', index=7,
      number=9, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\030\001'))),
    _descriptor.FieldDescriptor(
      name='size', full_name='naothmessages.CameraInfo.size', index=8,
      number=10, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\030\001'))),
    _descriptor.FieldDescriptor(
      name='memorysize', full_name='naothmessages.CameraInfo.memorysize', index=9,
      number=11, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\030\001'))),
    _descriptor.FieldDescriptor(
      name='cameraRollOffset', full_name='naothmessages.CameraInfo.cameraRollOffset', index=10,
      number=12, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\030\001'))),
    _descriptor.FieldDescriptor(
      name='cameraTiltOffset', full_name='naothmessages.CameraInfo.cameraTiltOffset', index=11,
      number=13, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\030\001'))),
    _descriptor.FieldDescriptor(
      name='correctionOffset', full_name='naothmessages.CameraInfo.correctionOffset', index=12,
      number=18, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\030\001'))),
    _descriptor.FieldDescriptor(
      name='headJointOffset', full_name='naothmessages.CameraInfo.headJointOffset', index=13,
      number=19, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\030\001'))),
    _descriptor.FieldDescriptor(
      name='transformation', full_name='naothmessages.CameraInfo.transformation', index=14,
      number=14, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\030\001'))),
    _descriptor.FieldDescriptor(
      name='openingAngleDiagonal', full_name='naothmessages.CameraInfo.openingAngleDiagonal', index=15,
      number=15, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='pixelSize', full_name='naothmessages.CameraInfo.pixelSize', index=16,
      number=16, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='focus', full_name='naothmessages.CameraInfo.focus', index=17,
      number=17, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=292,
  serialized_end=886,
)


_JOINTDATA = _descriptor.Descriptor(
  name='JointData',
  full_name='naothmessages.JointData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='position', full_name='naothmessages.JointData.position', index=0,
      number=1, type=1, cpp_type=5, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='stiffness', full_name='naothmessages.JointData.stiffness', index=1,
      number=2, type=1, cpp_type=5, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='dp', full_name='naothmessages.JointData.dp', index=2,
      number=3, type=1, cpp_type=5, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='ddp', full_name='naothmessages.JointData.ddp', index=3,
      number=4, type=1, cpp_type=5, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=888,
  serialized_end=961,
)


_SENSORJOINTDATA = _descriptor.Descriptor(
  name='SensorJointData',
  full_name='naothmessages.SensorJointData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='jointData', full_name='naothmessages.SensorJointData.jointData', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='temperature', full_name='naothmessages.SensorJointData.temperature', index=1,
      number=2, type=1, cpp_type=5, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='electricCurrent', full_name='naothmessages.SensorJointData.electricCurrent', index=2,
      number=3, type=1, cpp_type=5, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=963,
  serialized_end=1071,
)


_ULTRASOUNDRECEIVEDATA = _descriptor.Descriptor(
  name='UltraSoundReceiveData',
  full_name='naothmessages.UltraSoundReceiveData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='ultraSoundTimeStep', full_name='naothmessages.UltraSoundReceiveData.ultraSoundTimeStep', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='rawdata', full_name='naothmessages.UltraSoundReceiveData.rawdata', index=1,
      number=2, type=1, cpp_type=5, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='dataLeft', full_name='naothmessages.UltraSoundReceiveData.dataLeft', index=2,
      number=3, type=1, cpp_type=5, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='dataRight', full_name='naothmessages.UltraSoundReceiveData.dataRight', index=3,
      number=4, type=1, cpp_type=5, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1073,
  serialized_end=1178,
)


_FRAMEINFO = _descriptor.Descriptor(
  name='FrameInfo',
  full_name='naothmessages.FrameInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='frameNumber', full_name='naothmessages.FrameInfo.frameNumber', index=0,
      number=1, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='time', full_name='naothmessages.FrameInfo.time', index=1,
      number=2, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1180,
  serialized_end=1226,
)


_ROBOTINFO = _descriptor.Descriptor(
  name='RobotInfo',
  full_name='naothmessages.RobotInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='platform', full_name='naothmessages.RobotInfo.platform', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='bodyNickName', full_name='naothmessages.RobotInfo.bodyNickName', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='headNickName', full_name='naothmessages.RobotInfo.headNickName', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='bodyID', full_name='naothmessages.RobotInfo.bodyID', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='basicTimeStep', full_name='naothmessages.RobotInfo.basicTimeStep', index=4,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1228,
  serialized_end=1340,
)


_FSRDATA = _descriptor.Descriptor(
  name='FSRData',
  full_name='naothmessages.FSRData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='force', full_name='naothmessages.FSRData.force', index=0,
      number=1, type=1, cpp_type=5, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\030\001'))),
    _descriptor.FieldDescriptor(
      name='data', full_name='naothmessages.FSRData.data', index=1,
      number=2, type=1, cpp_type=5, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='valid', full_name='naothmessages.FSRData.valid', index=2,
      number=3, type=8, cpp_type=7, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\030\001'))),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1342,
  serialized_end=1403,
)


_ACCELEROMETERDATA = _descriptor.Descriptor(
  name='AccelerometerData',
  full_name='naothmessages.AccelerometerData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='legacyPackedData', full_name='naothmessages.AccelerometerData.legacyPackedData', index=0,
      number=1, type=1, cpp_type=5, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\030\001'))),
    _descriptor.FieldDescriptor(
      name='data', full_name='naothmessages.AccelerometerData.data', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='rawData', full_name='naothmessages.AccelerometerData.rawData', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1406,
  serialized_end=1546,
)


_GYROMETERDATA = _descriptor.Descriptor(
  name='GyrometerData',
  full_name='naothmessages.GyrometerData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='legacyPackedData', full_name='naothmessages.GyrometerData.legacyPackedData', index=0,
      number=1, type=1, cpp_type=5, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\030\001'))),
    _descriptor.FieldDescriptor(
      name='data', full_name='naothmessages.GyrometerData.data', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='rawData', full_name='naothmessages.GyrometerData.rawData', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='ref', full_name='naothmessages.GyrometerData.ref', index=3,
      number=4, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1549,
  serialized_end=1698,
)


_INERTIALSENSORDATA = _descriptor.Descriptor(
  name='InertialSensorData',
  full_name='naothmessages.InertialSensorData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='legacyPackedData', full_name='naothmessages.InertialSensorData.legacyPackedData', index=0,
      number=1, type=1, cpp_type=5, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\030\001'))),
    _descriptor.FieldDescriptor(
      name='data', full_name='naothmessages.InertialSensorData.data', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1700,
  serialized_end=1794,
)


_BUTTONDATA = _descriptor.Descriptor(
  name='ButtonData',
  full_name='naothmessages.ButtonData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='isPressed', full_name='naothmessages.ButtonData.isPressed', index=0,
      number=1, type=8, cpp_type=7, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='numOfFramesPressed', full_name='naothmessages.ButtonData.numOfFramesPressed', index=1,
      number=2, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='eventCounter', full_name='naothmessages.ButtonData.eventCounter', index=2,
      number=3, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _BUTTONDATA_BUTTONID,
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1797,
  serialized_end=2136,
)

_IMAGE.fields_by_name['cameraInfo'].message_type = _CAMERAINFO
_IMAGE.fields_by_name['format'].enum_type = _IMAGE_FORMAT
_IMAGE_FORMAT.containing_type = _IMAGE
_CAMERAINFO.fields_by_name['cameraID'].enum_type = CommonTypes__pb2._CAMERAID
_CAMERAINFO.fields_by_name['correctionOffset'].message_type = CommonTypes__pb2._DOUBLEVECTOR2
_CAMERAINFO.fields_by_name['headJointOffset'].message_type = CommonTypes__pb2._DOUBLEVECTOR2
_CAMERAINFO.fields_by_name['transformation'].message_type = CommonTypes__pb2._POSE3D
_SENSORJOINTDATA.fields_by_name['jointData'].message_type = _JOINTDATA
_ACCELEROMETERDATA.fields_by_name['data'].message_type = CommonTypes__pb2._DOUBLEVECTOR3
_ACCELEROMETERDATA.fields_by_name['rawData'].message_type = CommonTypes__pb2._DOUBLEVECTOR3
_GYROMETERDATA.fields_by_name['data'].message_type = CommonTypes__pb2._DOUBLEVECTOR3
_GYROMETERDATA.fields_by_name['rawData'].message_type = CommonTypes__pb2._DOUBLEVECTOR3
_INERTIALSENSORDATA.fields_by_name['data'].message_type = CommonTypes__pb2._DOUBLEVECTOR2
_BUTTONDATA_BUTTONID.containing_type = _BUTTONDATA
DESCRIPTOR.message_types_by_name['Image'] = _IMAGE
DESCRIPTOR.message_types_by_name['CameraInfo'] = _CAMERAINFO
DESCRIPTOR.message_types_by_name['JointData'] = _JOINTDATA
DESCRIPTOR.message_types_by_name['SensorJointData'] = _SENSORJOINTDATA
DESCRIPTOR.message_types_by_name['UltraSoundReceiveData'] = _ULTRASOUNDRECEIVEDATA
DESCRIPTOR.message_types_by_name['FrameInfo'] = _FRAMEINFO
DESCRIPTOR.message_types_by_name['RobotInfo'] = _ROBOTINFO
DESCRIPTOR.message_types_by_name['FSRData'] = _FSRDATA
DESCRIPTOR.message_types_by_name['AccelerometerData'] = _ACCELEROMETERDATA
DESCRIPTOR.message_types_by_name['GyrometerData'] = _GYROMETERDATA
DESCRIPTOR.message_types_by_name['InertialSensorData'] = _INERTIALSENSORDATA
DESCRIPTOR.message_types_by_name['ButtonData'] = _BUTTONDATA
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Image = _reflection.GeneratedProtocolMessageType('Image', (_message.Message,), dict(
  DESCRIPTOR = _IMAGE,
  __module__ = 'Framework_Representations_pb2'
  # @@protoc_insertion_point(class_scope:naothmessages.Image)
  ))
_sym_db.RegisterMessage(Image)

CameraInfo = _reflection.GeneratedProtocolMessageType('CameraInfo', (_message.Message,), dict(
  DESCRIPTOR = _CAMERAINFO,
  __module__ = 'Framework_Representations_pb2'
  # @@protoc_insertion_point(class_scope:naothmessages.CameraInfo)
  ))
_sym_db.RegisterMessage(CameraInfo)

JointData = _reflection.GeneratedProtocolMessageType('JointData', (_message.Message,), dict(
  DESCRIPTOR = _JOINTDATA,
  __module__ = 'Framework_Representations_pb2'
  # @@protoc_insertion_point(class_scope:naothmessages.JointData)
  ))
_sym_db.RegisterMessage(JointData)

SensorJointData = _reflection.GeneratedProtocolMessageType('SensorJointData', (_message.Message,), dict(
  DESCRIPTOR = _SENSORJOINTDATA,
  __module__ = 'Framework_Representations_pb2'
  # @@protoc_insertion_point(class_scope:naothmessages.SensorJointData)
  ))
_sym_db.RegisterMessage(SensorJointData)

UltraSoundReceiveData = _reflection.GeneratedProtocolMessageType('UltraSoundReceiveData', (_message.Message,), dict(
  DESCRIPTOR = _ULTRASOUNDRECEIVEDATA,
  __module__ = 'Framework_Representations_pb2'
  # @@protoc_insertion_point(class_scope:naothmessages.UltraSoundReceiveData)
  ))
_sym_db.RegisterMessage(UltraSoundReceiveData)

FrameInfo = _reflection.GeneratedProtocolMessageType('FrameInfo', (_message.Message,), dict(
  DESCRIPTOR = _FRAMEINFO,
  __module__ = 'Framework_Representations_pb2'
  # @@protoc_insertion_point(class_scope:naothmessages.FrameInfo)
  ))
_sym_db.RegisterMessage(FrameInfo)

RobotInfo = _reflection.GeneratedProtocolMessageType('RobotInfo', (_message.Message,), dict(
  DESCRIPTOR = _ROBOTINFO,
  __module__ = 'Framework_Representations_pb2'
  # @@protoc_insertion_point(class_scope:naothmessages.RobotInfo)
  ))
_sym_db.RegisterMessage(RobotInfo)

FSRData = _reflection.GeneratedProtocolMessageType('FSRData', (_message.Message,), dict(
  DESCRIPTOR = _FSRDATA,
  __module__ = 'Framework_Representations_pb2'
  # @@protoc_insertion_point(class_scope:naothmessages.FSRData)
  ))
_sym_db.RegisterMessage(FSRData)

AccelerometerData = _reflection.GeneratedProtocolMessageType('AccelerometerData', (_message.Message,), dict(
  DESCRIPTOR = _ACCELEROMETERDATA,
  __module__ = 'Framework_Representations_pb2'
  # @@protoc_insertion_point(class_scope:naothmessages.AccelerometerData)
  ))
_sym_db.RegisterMessage(AccelerometerData)

GyrometerData = _reflection.GeneratedProtocolMessageType('GyrometerData', (_message.Message,), dict(
  DESCRIPTOR = _GYROMETERDATA,
  __module__ = 'Framework_Representations_pb2'
  # @@protoc_insertion_point(class_scope:naothmessages.GyrometerData)
  ))
_sym_db.RegisterMessage(GyrometerData)

InertialSensorData = _reflection.GeneratedProtocolMessageType('InertialSensorData', (_message.Message,), dict(
  DESCRIPTOR = _INERTIALSENSORDATA,
  __module__ = 'Framework_Representations_pb2'
  # @@protoc_insertion_point(class_scope:naothmessages.InertialSensorData)
  ))
_sym_db.RegisterMessage(InertialSensorData)

ButtonData = _reflection.GeneratedProtocolMessageType('ButtonData', (_message.Message,), dict(
  DESCRIPTOR = _BUTTONDATA,
  __module__ = 'Framework_Representations_pb2'
  # @@protoc_insertion_point(class_scope:naothmessages.ButtonData)
  ))
_sym_db.RegisterMessage(ButtonData)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\n\031de.naoth.rc.core.messages'))
_CAMERAINFO.fields_by_name['focalLength'].has_options = True
_CAMERAINFO.fields_by_name['focalLength']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\030\001'))
_CAMERAINFO.fields_by_name['openingAngleWidth'].has_options = True
_CAMERAINFO.fields_by_name['openingAngleWidth']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\030\001'))
_CAMERAINFO.fields_by_name['openingAngleHeight'].has_options = True
_CAMERAINFO.fields_by_name['openingAngleHeight']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\030\001'))
_CAMERAINFO.fields_by_name['opticalCenterX'].has_options = True
_CAMERAINFO.fields_by_name['opticalCenterX']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\030\001'))
_CAMERAINFO.fields_by_name['opticalCenterY'].has_options = True
_CAMERAINFO.fields_by_name['opticalCenterY']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\030\001'))
_CAMERAINFO.fields_by_name['size'].has_options = True
_CAMERAINFO.fields_by_name['size']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\030\001'))
_CAMERAINFO.fields_by_name['memorysize'].has_options = True
_CAMERAINFO.fields_by_name['memorysize']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\030\001'))
_CAMERAINFO.fields_by_name['cameraRollOffset'].has_options = True
_CAMERAINFO.fields_by_name['cameraRollOffset']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\030\001'))
_CAMERAINFO.fields_by_name['cameraTiltOffset'].has_options = True
_CAMERAINFO.fields_by_name['cameraTiltOffset']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\030\001'))
_CAMERAINFO.fields_by_name['correctionOffset'].has_options = True
_CAMERAINFO.fields_by_name['correctionOffset']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\030\001'))
_CAMERAINFO.fields_by_name['headJointOffset'].has_options = True
_CAMERAINFO.fields_by_name['headJointOffset']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\030\001'))
_CAMERAINFO.fields_by_name['transformation'].has_options = True
_CAMERAINFO.fields_by_name['transformation']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\030\001'))
_FSRDATA.fields_by_name['force'].has_options = True
_FSRDATA.fields_by_name['force']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\030\001'))
_FSRDATA.fields_by_name['valid'].has_options = True
_FSRDATA.fields_by_name['valid']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\030\001'))
_ACCELEROMETERDATA.fields_by_name['legacyPackedData'].has_options = True
_ACCELEROMETERDATA.fields_by_name['legacyPackedData']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\030\001'))
_GYROMETERDATA.fields_by_name['legacyPackedData'].has_options = True
_GYROMETERDATA.fields_by_name['legacyPackedData']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\030\001'))
_INERTIALSENSORDATA.fields_by_name['legacyPackedData'].has_options = True
_INERTIALSENSORDATA.fields_by_name['legacyPackedData']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\030\001'))
# @@protoc_insertion_point(module_scope)
